@@ -25,18 +25,22 @@ import os
 import sys
 
 import eventlet
-from oslo_utils import encodeutils
-
-
-# Monkey patch socket, time, select, threads
 # NOTE(jokke): As per the eventlet commit
 # b756447bab51046dfc6f1e0e299cc997ab343701 there's circular import happening
 # which can be solved making sure the hubs are properly and fully imported
 # before calling monkey_patch(). This is solved in eventlet 0.22.0 but we
 # need to address it before that is widely used around.
 eventlet.hubs.get_hub()
-eventlet.patcher.monkey_patch(all=False, socket=True, time=True,
-                              select=True, thread=True, os=True)
+
+if os.name == 'nt':
+    # eventlet monkey patching the os module causes subprocess.Popen to fail
+    # on Windows when using pipes due to missing non-blocking IO support.
+    eventlet.patcher.monkey_patch(os=False)
+else:
+    eventlet.patcher.monkey_patch()
+
+from oslo_reports import guru_meditation_report as gmr
+from oslo_utils import encodeutils
 
 # If ../glance/__init__.py exists, add ../ to Python search path, so that
 # it will override what happens to be installed in /usr/(local/)lib/python...
@@ -55,21 +59,26 @@ from glance.common import config
 from glance.common import exception
 from glance.common import wsgi
 from glance import notifier
+from glance import version
 
 CONF = cfg.CONF
 CONF.import_group("profiler", "glance.common.wsgi")
 logging.register_options(CONF)
+wsgi.register_cli_opts()
 
-KNOWN_EXCEPTIONS = (RuntimeError,
-                    exception.WorkerCreationFailure,
-                    glance_store.exceptions.BadStoreConfiguration,
-                    ValueError)
+# NOTE(rosmaita): Any new exceptions added should preserve the current
+# error codes for backward compatibility.  The value 99 is returned
+# for errors not listed in this map.
+ERROR_CODE_MAP = {RuntimeError: 1,
+                  exception.WorkerCreationFailure: 2,
+                  glance_store.exceptions.BadStoreConfiguration: 3,
+                  ValueError: 4,
+                  cfg.ConfigFileValueError: 5}
 
 
 def fail(e):
-    global KNOWN_EXCEPTIONS
-    return_code = KNOWN_EXCEPTIONS.index(type(e)) + 1
     sys.stderr.write("ERROR: %s\n" % encodeutils.exception_to_unicode(e))
+    return_code = ERROR_CODE_MAP.get(type(e), 99)
     sys.exit(return_code)
 
 
@@ -79,6 +88,7 @@ def main():
         config.set_config_defaults()
         wsgi.set_eventlet_hub()
         logging.setup(CONF, 'glance')
+        gmr.TextGuruMeditation.setup_autorun(version)
         notifier.set_defaults()
 
         if CONF.profiler.enabled:
@@ -90,10 +100,18 @@ def main():
                 host=CONF.bind_host
             )
 
-        server = wsgi.Server(initialize_glance_store=True)
+        # NOTE(abhishekk): Added initialize_prefetcher KW argument to Server
+        # object so that prefetcher object should only be initialized in case
+        # of API service and ignored in case of registry. Once registry is
+        # removed this parameter should be removed as well.
+        initialize_prefetcher = False
+        if CONF.paste_deploy.flavor == 'keystone+cachemanagement':
+            initialize_prefetcher = True
+        server = wsgi.Server(initialize_glance_store=True,
+                             initialize_prefetcher=initialize_prefetcher)
         server.start(config.load_paste_app('glance-api'), default_port=9292)
         server.wait()
-    except KNOWN_EXCEPTIONS as e:
+    except Exception as e:
         fail(e)
 
 
