@@ -80,10 +80,16 @@ def main():
 
         config.parse_args()
         logging.setup(CONF, 'glance')
+        CONF.import_opt('enabled_backends', 'glance.common.wsgi')
 
-        glance_store.register_opts(config.CONF)
-        glance_store.create_stores(config.CONF)
-        glance_store.verify_default_store()
+        if CONF.enabled_backends:
+            glance_store.register_store_opts(CONF)
+            glance_store.create_multi_stores(CONF)
+            glance_store.verify_store()
+        else:
+            glance_store.register_opts(CONF)
+            glance_store.create_stores(CONF)
+            glance_store.verify_default_store()
 
         if CONF.restore and CONF.daemon:
             sys.exit("ERROR: The restore and daemon options should not be set "
