@@ -21,9 +21,11 @@ from six.moves import StringIO
 
 from glance.cmd import manage
 from glance.common import exception
+from glance.db.sqlalchemy import alembic_migrations
 from glance.db.sqlalchemy import api as db_api
 from glance.db.sqlalchemy import metadata as db_metadata
 from glance.tests import utils as test_utils
+from sqlalchemy.engine.url import make_url as sqlalchemy_make_url
 
 
 class TestManageBase(test_utils.BaseTestCase):
@@ -166,6 +168,17 @@ class TestManage(TestManageBase):
         self.output = StringIO()
         self.useFixture(fixtures.MonkeyPatch('sys.stdout', self.output))
 
+    def test_db_complex_password(self):
+        engine = mock.Mock()
+        # See comments in get_alembic_config; make an engine url with
+        # password characters that will be escaped, to ensure the
+        # resulting value makes it into alembic unaltered.
+        engine.url = sqlalchemy_make_url(
+            'mysql+pymysql://username:pw@%/!#$()@host:1234/dbname')
+        alembic_config = alembic_migrations.get_alembic_config(engine)
+        self.assertEqual(str(engine.url),
+                         alembic_config.get_main_option('sqlalchemy.url'))
+
     @mock.patch('glance.db.sqlalchemy.api.get_engine')
     @mock.patch(
         'glance.db.sqlalchemy.alembic_migrations.data_migrations.'
@@ -365,6 +378,9 @@ class TestManage(TestManageBase):
                       ' revision. But, current revisions are: test ',
                       exit.code)
 
+    @mock.patch(
+        'glance.db.sqlalchemy.alembic_migrations.data_migrations.'
+        'has_pending_migrations')
     @mock.patch(
         'glance.db.sqlalchemy.alembic_migrations.get_current_alembic_heads')
     @mock.patch(
@@ -373,13 +389,15 @@ class TestManage(TestManageBase):
     @mock.patch.object(manage.DbCommands, '_sync')
     def test_contract(self, mock_sync, mock_validate_engine,
                       mock_get_alembic_branch_head,
-                      mock_get_current_alembic_heads):
+                      mock_get_current_alembic_heads,
+                      mock_has_pending_migrations):
         engine = mock_validate_engine.return_value
         engine.engine.name = 'mysql'
         mock_get_current_alembic_heads.side_effect = ['pike_expand01',
                                                       'pike_contract01']
         mock_get_alembic_branch_head.side_effect = ['pike_contract01',
                                                     'pike_expand01']
+        mock_has_pending_migrations.return_value = False
         self.db.contract()
         mock_sync.assert_called_once_with(version='pike_contract01')
 
