@@ -22,7 +22,7 @@ from oslo_config import cfg
 import oslo_messaging
 import webob
 
-import glance.async
+import glance.async_
 from glance.common import exception
 from glance.common import timeutils
 import glance.context
@@ -44,7 +44,7 @@ class ImageStub(glance.domain.Image):
     def get_data(self, offset=0, chunk_size=None):
         return ['01234', '56789']
 
-    def set_data(self, data, size=None):
+    def set_data(self, data, size, backend=None):
         for chunk in data:
             pass
 
@@ -681,7 +681,7 @@ class TestTaskNotifications(utils.BaseTestCase):
         self.assertEqual(0, len(output_logs))
 
     def test_task_run_notification(self):
-        with mock.patch('glance.async.TaskExecutor') as mock_executor:
+        with mock.patch('glance.async_.TaskExecutor') as mock_executor:
             executor = mock_executor.return_value
             executor._run.return_value = mock.Mock()
             self.task_proxy.run(executor=mock_executor)
@@ -694,7 +694,7 @@ class TestTaskNotifications(utils.BaseTestCase):
 
     def test_task_run_notification_disabled(self):
         self.config(disabled_notifications=['task.run'])
-        with mock.patch('glance.async.TaskExecutor') as mock_executor:
+        with mock.patch('glance.async_.TaskExecutor') as mock_executor:
             executor = mock_executor.return_value
             executor._run.return_value = mock.Mock()
             self.task_proxy.run(executor=mock_executor)
