@@ -30,11 +30,23 @@ class VersionsTest(base.IsolatedUnitTest):
     def _get_versions_list(self, url):
         versions = [
             {
-                'id': 'v2.6',
+                'id': 'v2.9',
                 'status': 'CURRENT',
                 'links': [{'rel': 'self',
                            'href': '%s/v2/' % url}],
             },
+            {
+                'id': 'v2.7',
+                'status': 'SUPPORTED',
+                'links': [{'rel': 'self',
+                           'href': '%s/v2/' % url}],
+            },
+            {
+                'id': 'v2.6',
+                'status': 'SUPPORTED',
+                'links': [{'rel': 'self',
+                           'href': '%s/v2/' % url}],
+            },
             {
                 'id': 'v2.5',
                 'status': 'SUPPORTED',
@@ -71,18 +83,6 @@ class VersionsTest(base.IsolatedUnitTest):
                 'links': [{'rel': 'self',
                            'href': '%s/v2/' % url}],
             },
-            {
-                'id': 'v1.1',
-                'status': 'DEPRECATED',
-                'links': [{'rel': 'self',
-                           'href': '%s/v1/' % url}],
-            },
-            {
-                'id': 'v1.0',
-                'status': 'DEPRECATED',
-                'links': [{'rel': 'self',
-                           'href': '%s/v1/' % url}],
-            },
         ]
         return versions
 
@@ -142,27 +142,6 @@ class VersionNegotiationTest(base.IsolatedUnitTest):
         super(VersionNegotiationTest, self).setUp()
         self.middleware = version_negotiation.VersionNegotiationFilter(None)
 
-    def test_request_url_v1(self):
-        request = webob.Request.blank('/v1/images')
-        self.middleware.process_request(request)
-        self.assertEqual('/v1/images', request.path_info)
-
-    def test_request_url_v1_0(self):
-        request = webob.Request.blank('/v1.0/images')
-        self.middleware.process_request(request)
-        self.assertEqual('/v1/images', request.path_info)
-
-    def test_request_url_v1_1(self):
-        request = webob.Request.blank('/v1.1/images')
-        self.middleware.process_request(request)
-        self.assertEqual('/v1/images', request.path_info)
-
-    def test_request_accept_v1(self):
-        request = webob.Request.blank('/images')
-        request.headers = {'accept': 'application/vnd.openstack.images-v1'}
-        self.middleware.process_request(request)
-        self.assertEqual('/v1/images', request.path_info)
-
     def test_request_url_v2(self):
         request = webob.Request.blank('/v2/images')
         self.middleware.process_request(request)
@@ -203,14 +182,18 @@ class VersionNegotiationTest(base.IsolatedUnitTest):
         self.middleware.process_request(request)
         self.assertEqual('/v2/images', request.path_info)
 
-    def test_request_url_v2_7_unsupported(self):
+    def test_request_url_v2_7(self):
         request = webob.Request.blank('/v2.7/images')
-        resp = self.middleware.process_request(request)
-        self.assertIsInstance(resp, versions.Controller)
+        self.middleware.process_request(request)
+        self.assertEqual('/v2/images', request.path_info)
 
-    def test_request_url_v2_7_unsupported_EXPERIMENTAL(self):
-        request = webob.Request.blank('/v2.7/images')
-        self.config(enable_image_import=True)
+    def test_request_url_v2_9(self):
+        request = webob.Request.blank('/v2.9/images')
+        self.middleware.process_request(request)
+        self.assertEqual('/v2/images', request.path_info)
+
+    def test_request_url_v2_10_unsupported(self):
+        request = webob.Request.blank('/v2.10/images')
         resp = self.middleware.process_request(request)
         self.assertIsInstance(resp, versions.Controller)
 
